#define F_CPU 16000000
#include "lcd.h"
#include "font_data.h"
#include <inttypes.h>
#include <util/delay.h>

// API LCD

// Functie de initializare
void lcdInit ()
{
	LcdCMD_DDR = 0xff;
	LcdCMD_PORT = 0x00;
	
	LcdDATA_DDR = 0xff;
	LcdDATA_PORT = 0x00;
	
	LcdCMD_PORT |= (1 << LcdRES);
	
	WriteCommand(DISP_OFF);
	WriteCommand(START_LINE);
	WriteCommand(DISP_ON);
	
	clearl();
}

void setNewPage (unsigned char PageData)
{		
		PageData = PageData + SET_PAGE;
        WriteCommand(PageData);	
		
		WriteCommand(SET_ADDRESS); // reset column address to first
}								   // column again


// Clear
void clearl(void) 
{
	unsigned int i;

	column = 0;
	page = 0;

	WriteCommand(SET_ADDRESS);
	WriteCommand(SET_PAGE);
	
	for(i = 0; i < 1024; i++) {
		if(column == 128) {
			column = 0;
			page++;
			setNewPage(page);
		}
	
		cSelect(column);						
										
		DataMode(); 						
		LcdDATA_PORT = 0b00000000; 				// clear the pixels
		LocknLoad();						

		column++;     						// increase column (maximum 128).
	}
 
	// reset global variables
	column = 0;							// reset column and page to beginning
	page = 0;
 
	// reset LCD registers

	WriteCommand(SET_ADDRESS);				// reset column and page to the beginning
	WriteCommand(SET_PAGE);
 }
 
 void DataMode (void)
{   
  //set pins for data mode
	LcdCMD_PORT |= LcdRS;
	LcdCMD_PORT &= ~(1<<LcdRW); 
}

void WriteCommand(uint8_t cmd) {

		LcdCMD_PORT |= (1<<LcdCS1);   //Enable CS1 & CS2
        LcdCMD_PORT|= (1<<LcdCS2);   
        		
        LcdCMD_PORT &= ~(1<<LcdRW);   //Clear RW & RS for Command mode   
		LcdCMD_PORT &= ~(1<<LcdRS);   
		
    LcdDATA_PORT = cmd;   // write command
    LocknLoad();
}

void cSelect(uint8_t col)
{

if (col <= 63){
	LcdCMD_PORT |= (1<<LcdCS1);   //Enable CS1 
	LcdCMD_PORT &= ~(1<<LcdCS2);  //Disable CS2 	
	}
	
	if (col >= 64){
	LcdCMD_PORT |= (1<<LcdCS2);   //Enable CS2 
	LcdCMD_PORT &= ~(1<<LcdCS1);  //Disable CS1 	
	}

}

void LocknLoad (void)
{
    LcdCMD_PORT |= (1<<LcdE);   //Output - Data Load
	//delay
   _delay_us(4);
   
    LcdCMD_PORT &= ~(1<<LcdE);  //Latch - Data Lock
	//delay
   _delay_us(4);
}

void setXY(uint8_t x, uint8_t y) {

	uint8_t cmd;
	
	if(x > 127) x = 0;				// validate coordinates
	if(y > 63)  y = 0;								
	
	page = y/8;
	column = x;
	cSelect(column); 				// select the chip based on the column number
									// column is a global variable it's contents are used
									// by the ascii driver lcd_putch()
	
	
	cmd = SET_ADDRESS | x;	
	WriteCommand(cmd);
	
	cmd = SET_PAGE | page;			// set page, or y address on the selected chip
	WriteCommand(cmd);

}

void lcd_writech(uint8_t ch)  {

		

			// make sure the characters to print are within the ascii range values
			// as specified in the font bitmap file
			// if not they are modified to the outermost limits
			if (ch < ASCIIL) ch = ASCIIL;  	
			if (ch > ASCIIH) ch = ASCIIH;

			
			int base;	//HOLDING A CHAR VALUE **THIS MUST BE A INT AND NOT uint8_t
			uint8_t i, p, cmd;
  
			uint8_t tmppage=page;							// temporary page counter
			uint8_t tmpcol=column;							// temporary column counter for one character
			uint8_t EOL;	

			EOL = column + CHAR_FW;	
			base = (ch - ASCIIL) * CHAR_PG * CHAR_FW; 		
															
															
															
															

			if (EOL > 128){	
			
				column = 0;
				tmpcol = 0;
				cSelect(column); 							// selects the first half of the
															// lcd, column 0
				WriteCommand(SET_ADDRESS); 					// reset column address to first column in LCD as well	
			
				page = page + CHAR_PG;						// next line, number of pages down
				tmppage=page;
				cmd = SET_PAGE | page;						
				WriteCommand(cmd);
				
			}
								
			for (p=0;p<CHAR_PG;p++){	
			
				for(i = 0; i < CHAR_FW ;i++){ 				// CHAR_FW number of bytes for each character	
													
	
				cSelect(tmpcol);	
					DataMode(); 							// prepare mode to send a byte of data
					LcdDATA_PORT = pgm_read_byte(&basicfont[base++]); 	// send data from pgm space to the data port				
					LocknLoad();							// load data and relatch
				
					tmpcol++;
				}

				if(CHAR_PG>1){								// if a  multipage character
				   
				  tmppage = tmppage + 1;

				  cmd = SET_PAGE | tmppage;					// set next page for the multi-page character
				  WriteCommand(cmd);				  
				
				  cmd = SET_ADDRESS | column;				// reset the column back to the starting point
				  WriteCommand(cmd);
				  tmpcol = column;	
				  
				}

			}
			
			if ((column + CHAR_SPACE) < 128){											
				column = column + CHAR_FW + CHAR_SPACE;		// next character position
				cmd = SET_ADDRESS | column;				    // reset the column to the next character position
				WriteCommand(cmd);												  
			}
						
		
			if(CHAR_PG>1){									// if a  multipage character				  
				cmd = SET_PAGE | page;						// return back to the starting page
				WriteCommand(cmd);			  				  
			}
			
				
}

void printl(const char* str) {
  while(*str) {
    lcd_writech(*str++);
  }
}






