//#ifndef __LCD_H
//#define __LCD_H

#include <avr/pgmspace.h>
#include <avr\io.h>
#include <avr\interrupt.h>

/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRC			// Portul pe care conectam firele de date la LCD
#define LcdDATA_PORT	PORTC
#define LcdDATA_PIN		PINC

#define LcdCMD_DDR		DDRD			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTD
#define LcdCMD_PIN		PIND

#define LcdRS			 PD7			    // Pinul pentru selectare operatie (LCD)
#define LcdRW			 PD6				// Pinul pentru Read/ Write (LCD)
#define LcdE			 PD5				// Pinul de Enable (LCD)

#define LcdCS1			 PD4				// Pinul de CS1 (LCD)
#define LcdCS2			 PD3				// Pinul de CS2 (LCD)
#define LcdRES			 PD2				// Pinul de RESET

// LCD Commands
#define START_LINE		0b11000000
#define SET_ADDRESS		0b01000000	// column address to 0
#define SET_PAGE		0b10111000	// page address to 0
#define DISP_ON			0b00111111
#define DISP_OFF		0b00111110

// global variables
unsigned char column = 0;
unsigned char page = 0;

/*************
 * API LCD   *
 *************/
// fuction prototypes
void cSelect(uint8_t col);				// Select the controller chip depending on column number

// command pair
void CommandMode (void);				// lcd data port to accept commands
void WriteCommand(uint8_t cmd);			// send a command to the controllers

// data pair
void DataMode (void);					// lcd data port to accept pixel data
void LocknLoad (void);					// load the data buffer and latch for next byte

// initialise lcd
void lcdInit (void);					// initialise lcd
void clearl (void);						// clear lcd						

void setNewPage (unsigned char PageData);

// set cursor position
void setXY(uint8_t x, uint8_t y);


// font and text display						
void lcd_writech(uint8_t ch); 			// main character driver
void printl(const char* str);			// lcd user printing function
